from clypi import ClypiException

from .styler import error


class SimpleRecorderError(ClypiException):
    """Base class for all SimpleRecorder exceptions."""

    def __init__(self, message: str):
        super().__init__(error(message))
        self.raw_message = message



from datetime import datetime

import obsws_python as obsws
from clypi import Command, Positional, arg
from typing_extensions import override

from .errors import SimpleRecorderError
from .styler import highlight


class Start(Command):
    """Start recording."""

    filename: Positional[str] = arg(
        default="default_name",
        help="Name of the recording",
        prompt="Enter the name for the recording",
    )
    host: str = arg(inherited=True)
    port: int = arg(inherited=True)
    password: str = arg(inherited=True)

    @staticmethod
    def get_timestamp():
        return datetime.now().strftime("%Y-%m-%d %H-%M-%S")

    @override
    async def run(self):
        if not self.filename:
            raise SimpleRecorderError("Recording name cannot be empty.")

        with obsws.ReqClient(
            host=self.host, port=self.port, password=self.password
        ) as client:
            resp = client.get_record_status()
            if resp.output_active:
                raise SimpleRecorderError("Recording is already active.")

            filename = f"{self.filename} {self.get_timestamp()}"
            client.set_profile_parameter(
                "Output",
                "FilenameFormatting",
                filename,
            )
            client.start_record()
            print(f"Recording started with filename: {highlight(filename)}")



import logging

from clypi import ClypiConfig, ClypiException, Command, arg, configure
from typing_extensions import override

from .errors import SimpleRecorderError
from .gui import SimpleRecorderWindow
from .start import Start
from .stop import Stop

logger = logging.getLogger(__name__)

config = ClypiConfig(
    nice_errors=(SimpleRecorderError,),
)
configure(config)

themes = [
    "Light Purple",
    "Neutral Blue",
    "Reds",
    "Sandy Beach",
    "Kayak",
    "Light Blue 2",
]


def theme_parser(value: str) -> str:
    """Parse the theme argument."""
    if value not in themes:
        raise ClypiException(
            f"Invalid theme: {value}. Available themes: {', '.join(themes)}"
        )
    return value


class SimpleRecorder(Command):
    subcommand: Start | Stop | None = None
    host: str = arg(default="localhost", env="OBS_HOST", help="OBS WebSocket host")
    port: int = arg(default=4455, env="OBS_PORT", help="OBS WebSocket port")
    password: str | None = arg(
        default=None, env="OBS_PASSWORD", help="OBS WebSocket password"
    )
    theme: str = arg(
        default="Reds",
        parser=theme_parser,
        env="OBS_THEME",
        help=f"GUI theme ({', '.join(themes)})",
    )
    debug: bool = arg(
        default=False,
        env="DEBUG",
        help="Enable debug logging",
    )

    @override
    async def run(self):
        """Run the Simple Recorder GUI."""
        if self.debug:
            logging.basicConfig(level=logging.DEBUG)

        window = SimpleRecorderWindow(self.host, self.port, self.password, self.theme)
        await window.run()


def run():
    """Run the application."""
    SimpleRecorder.parse().start()



import logging

import FreeSimpleGUI as fsg

from .errors import SimpleRecorderError
from .start import Start
from .stop import Stop

logger = logging.getLogger(__name__)


class SimpleRecorderWindow(fsg.Window):
    def __init__(self, host, port, password, theme):
        self.logger = logger.getChild(self.__class__.__name__)
        self.host = host
        self.port = port
        self.password = password
        fsg.theme(theme)

        layout = [
            [fsg.Text("Enter recording filename:")],
            [fsg.InputText("", key="-FILENAME-")],
            [fsg.Button("Start Recording"), fsg.Button("Stop Recording")],
            [fsg.Text("Status: Not started", key="-OUTPUT-")],
        ]
        super().__init__("Simple Recorder", layout, finalize=True)
        self["-FILENAME-"].bind("<Return>", " || RETURN")
        self["Start Recording"].bind("<Return>", " || RETURN")
        self["Stop Recording"].bind("<Return>", " || RETURN")

    async def run(self):
        while True:
            event, values = self.read()
            self.logger.debug(f"Event: {event}, Values: {values}")
            if event == fsg.WIN_CLOSED:
                break

            match e := event.split(" || "):
                case ["Start Recording"] | ["Start Recording" | "-FILENAME-", "RETURN"]:
                    try:
                        await Start(
                            filename=values["-FILENAME-"],
                            host=self.host,
                            port=self.port,
                            password=self.password,
                        ).run()
                        self["-OUTPUT-"].update(
                            "Recording started successfully", text_color="green"
                        )
                    except SimpleRecorderError as e:
                        self["-OUTPUT-"].update(
                            f"Error: {e.raw_message}", text_color="red"
                        )

                case ["Stop Recording"] | ["Stop Recording", "RETURN"]:
                    try:
                        await Stop(
                            host=self.host, port=self.port, password=self.password
                        ).run()
                        self["-OUTPUT-"].update(
                            "Recording stopped successfully", text_color="green"
                        )
                    except SimpleRecorderError as e:
                        self["-OUTPUT-"].update(
                            f"Error: {e.raw_message}", text_color="red"
                        )

                case _:
                    self.logger.warning(f"Unhandled event: {e}")

        self.close()



import obsws_python as obsws
from clypi import Command, arg
from typing_extensions import override

from .errors import SimpleRecorderError
from .styler import highlight


class Stop(Command):
    """Stop recording."""

    host: str = arg(inherited=True)
    port: int = arg(inherited=True)
    password: str = arg(inherited=True)

    @override
    async def run(self):
        with obsws.ReqClient(
            host=self.host, port=self.port, password=self.password
        ) as client:
            resp = client.get_record_status()
            if not resp.output_active:
                raise SimpleRecorderError("Recording is not active.")

            client.stop_record()
            print(highlight("Recording stopped successfully."))
